#include "M5LCD.h"

/*
 * M5LCD classe
 */

M5LCD::M5LCD() : _current_page(0), _logs() , _debug_loc_y(0), _components_status({COMPONENT_KO, COMPONENT_KO, COMPONENT_KO, COMPONENT_KO, COMPONENT_KO}) {
    this->_product_id = std::string("");
    this->_product_label = std::string("");
    this->_last_nfc = std::string("");
    this->_servo_current_position = std::string("");
    this->_dolibarr_msg = std::string("");
    M5.begin();
    M5.Power.begin();
    M5.lcd.setBrightness(100);
    this->update_page();
}

void M5LCD::update() {
    M5.update();
    if (M5.BtnB.wasReleased() != 0) {
        this->_current_page = (this->_current_page + 1) % LCD_PAGES;
        this->update_page();
    }
    if (this->_current_page == DEBUG_SCREEN && M5.BtnA.wasReleased() != 0 && this->_debug_loc_y < 0) {
        this->_debug_loc_y++;
        this->update_page();
    }
    if (this->_current_page == DEBUG_SCREEN && M5.BtnC.wasReleased() != 0) {
        this->_debug_loc_y--;
        this->update_page();
    }
}

void M5LCD::update_page() const {
    M5.Lcd.clear();
    switch (this->_current_page) {
        case 0:
            this->show_dashboard();
            break;
        case 1:
            this->show_debug();
            break;
        case 2:
            this->show_config();
            break;
    }
    this->update_pagination();
}

void M5LCD::display_error(const char *str) const {
    M5.Lcd.setTextColor(WHITE, RED);
    M5.Lcd.setTextSize(1);
    M5.Lcd.setCursor(0, 40);
    M5.Lcd.println(str);
}

void M5LCD::display_message(const char *str) const {
}

void M5LCD::display_warning(const char *str) const {
}

std::string get_status(AvailableComponentsStatus status) {
    return status == COMPONENT_OK ? "OK" : "KO";
}

void draw_component_stats(int x, int y, int fx, int fy, int fw, int fh,  const char *name, AvailableComponentsStatus status) {
    M5.Lcd.setTextSize(2);
    if (status == COMPONENT_OK) {
        M5.Lcd.setTextColor(WHITE, GREEN);
    } else {
        M5.Lcd.setTextColor(WHITE, RED);
    }
    if (status == COMPONENT_OK) {
        M5.Lcd.fillRect(fx, fy, fw, fh, GREEN);
    } else {
        M5.Lcd.fillRect(fx, fy, fw, fh, RED);
    }
    M5.Lcd.setCursor(x, y);
    M5.Lcd.printf("%s %s", name, get_status(status).c_str());
}

void M5LCD::show_dashboard() const {
    draw_component_stats(10, 18, 0, 0, 100, 50, "NFC", _components_status.nfc);
    draw_component_stats(150, 18, 110, 0, 210, 50, "DOLIBARR", _components_status.dolibarr);
    draw_component_stats(8, 78, 0, 60, 100, 50, "WIFI", _components_status.wifi);
    draw_component_stats(112, 78, 110, 60, 100, 50, "SERVO", _components_status.servo);
    draw_component_stats(230, 78, 220, 60, 100, 50, "GRBL", _components_status.grbl);

    M5.Lcd.drawRect(0, 120, 320, 100, BLUE);
    M5.Lcd.setTextColor(WHITE, BLACK);
    M5.Lcd.setTextSize(2);
    if (!this->_last_nfc.empty()) {
        M5.Lcd.setCursor(10, 130);
        M5.Lcd.printf("NFC: %s", this->_last_nfc.c_str());
    }
    if (!this->_product_id.empty() && !this->_product_label.empty()) {
        M5.Lcd.setCursor(10, 150);
        M5.Lcd.printf("Produit: %s - %s", this->_product_label.c_str(), this->_product_id.c_str());
    }
    if (!this->_dolibarr_msg.empty()) {
        M5.Lcd.setCursor(10, 170);
        M5.Lcd.printf("Produit: %s", this->_last_nfc.c_str());
    }
    if (!this->_servo_current_position.empty()) {
        M5.Lcd.setCursor(10, 190);
        M5.Lcd.printf("SERVO Posistion: %s", this->_servo_current_position.c_str());
    }

    M5.Lcd.setTextSize(1);
    M5.Lcd.setCursor(0, 230);
    M5.Lcd.printf("%s - %s", APP_TITLE, APP_VERSION);
}

void M5LCD::show_debug() const {
    M5.Lcd.setTextColor(WHITE, BLACK);
    M5.Lcd.setTextSize(1);
    int i = 0;
    for (auto val : this->_logs) {
        M5.Lcd.setCursor(0, i + (this->_debug_loc_y * 10));
        M5.Lcd.printf("[%s] - %s", val.get_datetime_format(), val.get_message().c_str());
        i+=10;
    }
}

void M5LCD::update_pagination() const {
    M5.Lcd.setTextSize(1);
    M5.Lcd.setTextColor(WHITE, BLACK);
    M5.Lcd.setCursor(302, 230);
    M5.Lcd.printf("%d/%d", this->_current_page+1, LCD_PAGES);
}

void M5LCD::add_log(const char* str) {
    this->_logs.emplace_back(str);
    if (this->_current_page == DEBUG_SCREEN) {
        this->show_debug();
    }
}

void M5LCD::show_config() const {

}

ComponentsStatus M5LCD::get_components() {
    return this->_components_status;
}

void M5LCD::set_wifi_status(AvailableComponentsStatus status) {
    this->_components_status.wifi = status;
    this->update_dashboard();
}

void M5LCD::set_nfc_status(AvailableComponentsStatus status) {
    this->_components_status.nfc = status;
    this->update_dashboard();
}

void M5LCD::set_grbl_status(AvailableComponentsStatus status) {
    this->_components_status.grbl = status;
    this->update_dashboard();
}

void M5LCD::set_servo(AvailableComponentsStatus status) {
    this->_components_status.servo = status;
    this->update_dashboard();
}

void M5LCD::set_dolibarr_status(AvailableComponentsStatus status) {
    this->_components_status.dolibarr = status;
    this->update_dashboard();
}

void M5LCD::update_dashboard() const {
    if (this->_current_page == DASHBOARD_SCREEN) {
        this->show_dashboard();
    }
}


void M5LCD::set_dolibarr_message(std::string str) {
    this->_dolibarr_msg = str;
    this->update_dashboard();
}

void M5LCD::set_nfc_message(std::string str) {
    this->_last_nfc = str;
    this->update_dashboard();
}

void M5LCD::set_product_id(std::string str) {
    this->_product_id = str;
    this->update_dashboard();
}

void M5LCD::set_servo_message(std::string str) {
    this->_servo_current_position = str;
    this->update_dashboard();
}

void M5LCD::set_product_label(std::string str) {
    this->_product_label = str;
    this->update_dashboard();

}

/*
 * LogMessage classe
 */

LogMessage::LogMessage(std::string log) {
    this->log = log;
    this->datetime = time(nullptr);
}

std::string LogMessage::get_message() const {
    return this->log;
}

const char *LogMessage::get_datetime_format() const {
    void *buff = malloc(20 * sizeof(char));
    strftime((char*) buff, 20, "%H:%M:%S", localtime(&this->datetime));
    return (char*) buff;
}
